==> nodejs_interop-0.7.0.tar.gz: lib/src/io/process/process.dart <==
part of nodejs_interop.io;

/// The means to execute a program.
///
/// Use the static [start] and [run] methods to start a new process. The [run] method executes the process non-interactively to completion.
/// In contrast, the [start] method allows your code to interact with the running process.
class Process implements io.Process {

  /// A reference to the `child_process` module.
  static final ChildProcessModule _childProcess = require('child_process');

  /// The underlying child process.
  final ChildProcess _child;

  /// Creates a new process.
  Process._internal(this._child):
    stderr = convertToStream(_child.stderr),
    stdout = convertToStream(_child.stdout);

  /// Kills the process with identifier [pid]. Where possible, sends the [signal] to the process.
  ///
  /// Returns `true` if the [signal] is successfully delivered to the process with identifier [pid].
  /// Otherwise the signal could not be sent, usually meaning that the process is already dead.
  static bool killPid(int pid, [io.ProcessSignal signal = io.ProcessSignal.SIGTERM]) {
    try {
      process.kill(pid, signal.toString());
      return true;
    }

    on JSError {
      return false;
    }
  }

  /// Starts a process and runs it non-interactively to completion.
  ///
  /// Returns a [Future] that completes with the [ProcessResult] of running the process, i.e. exit code, standard out and standard in.
  static Future<io.ProcessResult> run(String executable, List<String> arguments, {
    Map<String, String> environment,
    bool includeParentEnvironment = true,
    bool runInShell = false,
    Encoding stderrEncoding = UTF8,
    Encoding stdoutEncoding = UTF8,
    String workingDirectory
  }) {
    var env = new Map<String, String>.from(includeParentEnvironment ? const Platform().environment : const {})
      ..addAll(environment ?? const {});

    var options = new SpawnOptions(
      cwd: workingDirectory ?? Directory.current.path,
      encoding: BufferEncoding.fromEncoding(stdoutEncoding),
      env: jsify(env),
      shell: runInShell
    );

    var completer = new Completer<io.ProcessResult>();
    var child = _childProcess.spawn(executable, arguments, options);
    child
      .on('error', allowInterop((error) => completer.completeError(new io.ProcessException(executable, arguments, error.message))))
      .on('exit', allowInterop((code, signal) {
        var stderr = child.stderr.read();
        if (stderr != null && stderrEncoding != null) stderr = stderrEncoding.decode(stderr);

        var stdout = child.stdout.read();
        if (stdout != null && stdoutEncoding != null) stdout = stdoutEncoding.decode(stdout);

        completer.complete(new io.ProcessResult(child.pid, code, stdout, stderr));
      }));

    return completer.future;
  }

  /// Starts a process and runs it to completion.
  /// This is a synchronous call and will block until the child process terminates.
  ///
  /// Returns a [ProcessResult] with the result of running the process, i.e. exit code, standard out and standard in.
  static io.ProcessResult runSync(String executable, List<String> arguments, {
    Map<String, String> environment,
    bool includeParentEnvironment = true,
    bool runInShell = false,
    Encoding stderrEncoding = UTF8,
    Encoding stdoutEncoding = UTF8,
    String workingDirectory
  }) {
    try {
      var env = new Map<String, String>.from(includeParentEnvironment ? const Platform().environment : const {})
        ..addAll(environment ?? const {});

      var result = _childProcess.spawnSync(executable, arguments, new SpawnOptions(
        cwd: workingDirectory ?? Directory.current.path,
        encoding: BufferEncoding.fromEncoding(stdoutEncoding),
        env: jsify(env),
        shell: runInShell
      ));

      if (result.error != null) throw new io.ProcessException(executable, arguments, result.error.message);
      return new io.ProcessResult(result.pid, result.status, result.stdout, result.stderr);
    }

    on JSError catch (error) {
      throw new io.ProcessException(executable, arguments, error.message);
    }
  }

  /// Starts a process. Returns a [Future] that completes with a [Process] instance when the process has been successfully started.
  /// That object can be used to interact with the process. If the process cannot be started the returned future completes with an exception.
  static Future<Process> start(String executable, List<String> arguments, {
    Map<String, String> environment,
    bool includeParentEnvironment = true,
    io.ProcessStartMode mode = io.ProcessStartMode.NORMAL,
    bool runInShell = false, String workingDirectory
  }) async => throw new UnimplementedError();

  /// A [Future] which completes with the exit code of the process when the process completes.
  @override
  Future<int> get exitCode {
    var completer = new Completer<int>();
    _child.on('exit', allowInterop((code, signal) {
      print('exitCode2: $code / $signal');
      code ??= int.parse(signal, radix: 10); // TODO
      completer.complete(code ?? -1); // TODO compute code from signal instead of -1
    }));

    return completer.future;
  }

  /// The process identifier (PID) of this process.
  @override
  int get pid => _child.pid;

  /// The standard error stream of the process as a [Stream].
  @override
  final Stream<List<int>> stderr;

  /// The standard input stream of the process as an [IOSink].
  @override
  IOSink get stdin => throw new UnimplementedError(); // TODO: convertToIOSink(_child.stdin)

  /// The standard output stream of the process as a [Stream].
  @override
  final Stream<List<int>> stdout;

  /// Kills this process. Where possible, sends the [signal] to the process.
  ///
  /// Returns `true` if the [signal] is successfully delivered to the process with identifier [pid].
  /// Otherwise the signal could not be sent, usually meaning that the process is already dead.
  @override
  bool kill([io.ProcessSignal signal = io.ProcessSignal.SIGTERM]) {
    try {
      _child.kill(signal.toString());
      return true;
    }

    on JSError {
      return false;
    }
  }
}

==> nodejs_interop-0.8.0.tar.gz: lib/src/io/process/process.dart <==
part of nodejs_interop.io;

/// The global exit code for the Node.js VM.
int get exitCode => hasProperty(process, 'exitCode') ? process.exitCode : 0;
set exitCode(int value) => process.exitCode = value;

/// The PID of the current process.
int get pid => process.pid;

/// Exits the [Node.js](https://nodejs.org) process immediately with the given exit code.
void exit(int code) => process.exit(code);

/// Sleeps for the duration specified in [duration].
void sleep(Duration duration) {
  if (duration.inMilliseconds < 0) throw new ArgumentError('The duration cannot be negative');
  var timestamp = new DateTime.now().add(new Duration(milliseconds: duration.inMilliseconds)).millisecondsSinceEpoch;
  var arguments = ['--eval', 'setTimeout(() => null, $timestamp - Date.now())'];
  loadLibrary<ChildProcessModule>('child_process').execFileSync(const Platform().resolvedExecutable, arguments);
}

/// The means to execute a program.
///
/// Use the static [start] and [run] methods to start a new process. The [run] method executes the process non-interactively to completion.
/// In contrast, the [start] method allows your code to interact with the running process.
class Process implements io.Process {

  /// A reference to the `child_process` module.
  static final ChildProcessModule _childProcess = require('child_process');

  /// The underlying child process.
  final ChildProcess _child;

  /// Creates a new process.
  Process._internal(this._child):
    stderr = asStream(_child.stderr),
    stdout = asStream(_child.stdout);

  /// Kills the process with identifier [pid]. Where possible, sends the [signal] to the process.
  ///
  /// Returns `true` if the [signal] is successfully delivered to the process with identifier [pid].
  /// Otherwise the signal could not be sent, usually meaning that the process is already dead.
  static bool killPid(int pid, [io.ProcessSignal signal = io.ProcessSignal.SIGTERM]) {
    try {
      process.kill(pid, signal.toString());
      return true;
    }

    on JSError {
      return false;
    }
  }

  /// Starts a process and runs it non-interactively to completion.
  ///
  /// Returns a [Future] that completes with the [ProcessResult] of running the process, i.e. exit code, standard out and standard in.
  static Future<io.ProcessResult> run(String executable, List<String> arguments, {
    Map<String, String> environment,
    bool includeParentEnvironment = true,
    bool runInShell = false,
    Encoding stderrEncoding = UTF8,
    Encoding stdoutEncoding = UTF8,
    String workingDirectory
  }) {
    var env = new Map<String, String>.from(includeParentEnvironment ? const Platform().environment : const <String, String>{})
      ..addAll(environment ?? const {});

    var options = new SpawnOptions(
      cwd: workingDirectory ?? Directory.current.path,
      encoding: BufferEncoding.fromEncoding(stdoutEncoding),
      env: jsify(env),
      shell: runInShell
    );

    var completer = new Completer<io.ProcessResult>();
    var child = _childProcess.spawn(executable, arguments, options);
    child
      .on('error', allowInterop((error) => completer.completeError(new io.ProcessException(executable, arguments, error.message))))
      .on('exit', allowInterop((code, signal) {
        var stderr = child.stderr.read();
        if (stderr != null && stderrEncoding != null) stderr = stderrEncoding.decode(stderr);

        var stdout = child.stdout.read();
        if (stdout != null && stdoutEncoding != null) stdout = stdoutEncoding.decode(stdout);

        completer.complete(new io.ProcessResult(child.pid, code, stdout, stderr));
      }));

    return completer.future;
  }

  /// Starts a process and runs it to completion.
  /// This is a synchronous call and will block until the child process terminates.
  ///
  /// Returns a [ProcessResult] with the result of running the process, i.e. exit code, standard out and standard in.
  static io.ProcessResult runSync(String executable, List<String> arguments, {
    Map<String, String> environment,
    bool includeParentEnvironment = true,
    bool runInShell = false,
    Encoding stderrEncoding = UTF8,
    Encoding stdoutEncoding = UTF8,
    String workingDirectory
  }) {
    try {
      var env = new Map<String, String>.from(includeParentEnvironment ? const Platform().environment : const <String, String>{})
        ..addAll(environment ?? const {});

      var result = _childProcess.spawnSync(executable, arguments, new SpawnOptions(
        cwd: workingDirectory ?? Directory.current.path,
        encoding: BufferEncoding.fromEncoding(stdoutEncoding),
        env: jsify(env),
        shell: runInShell
      ));

      if (result.error != null) throw new io.ProcessException(executable, arguments, result.error.message);
      return new io.ProcessResult(result.pid, result.status, result.stdout, result.stderr);
    }

    on JSError catch (error) {
      throw new io.ProcessException(executable, arguments, error.message);
    }
  }

  /// Starts a process. Returns a [Future] that completes with a [Process] instance when the process has been successfully started.
  /// That object can be used to interact with the process. If the process cannot be started the returned future completes with an exception.
  static Future<Process> start(String executable, List<String> arguments, {
    Map<String, String> environment,
    bool includeParentEnvironment = true,
    io.ProcessStartMode mode = io.ProcessStartMode.NORMAL,
    bool runInShell = false, String workingDirectory
  }) async => throw new UnimplementedError('Process.start');

  /// A [Future] which completes with the exit code of the process when the process completes.
  @override
  Future<int> get exitCode {
    var completer = new Completer<int>();
    _child.on('exit', allowInterop((code, signal) {
      print('exitCode2: $code / $signal');
      code ??= int.parse(signal, radix: 10); // TODO
      completer.complete(code ?? -1); // TODO compute code from signal instead of -1
    }));

    return completer.future;
  }

  /// The process identifier (PID) of this process.
  @override
  int get pid => _child.pid;

  /// The standard error stream of the process as a [Stream].
  @override
  final Stream<List<int>> stderr;

  /// The standard input stream of the process as an [IOSink].
  @override
  IOSink get stdin => throw new UnimplementedError('Process.stdin'); // TODO: convertToIOSink(_child.stdin)

  /// The standard output stream of the process as a [Stream].
  @override
  final Stream<List<int>> stdout;

  /// Kills this process. Where possible, sends the [signal] to the process.
  ///
  /// Returns `true` if the [signal] is successfully delivered to the process with identifier [pid].
  /// Otherwise the signal could not be sent, usually meaning that the process is already dead.
  @override
  bool kill([io.ProcessSignal signal = io.ProcessSignal.SIGTERM]) {
    try {
      _child.kill(signal.toString());
      return true;
    }

    on JSError {
      return false;
    }
  }

  /// Returns a string representation of this object.
  @override
  String toString() => 'Process($pid)';
}
